import 'dart:async';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter_menu_food_subs/main.dart';

class CuidaiRoutes {
  CuidaiRoutes._();

  static const String programDecision = "programs.decision";
  static const String programExplorer = "programs.discover";
  static const String programSearch = "programs.search";
  static const String exerciseExplorer = "trainings.discover";
  static const String meditationExplorer = "meditations.discover";
  static const String chatBotLogin = "chatbot?chatbotId=152&title=Vitat&retry=true";
  static const String chatBot = "chatbot";
  static const String programJourney = "programs.journey";
}

class CuidaiCommunication {
  late Function _onRoute;

  CuidaiCommunication({required onRoute(String route)}) {
    _onRoute = onRoute;
  }

  Future<dynamic> nativeMethodCallHandler(MethodCall call) async {
    switch (call.method) {
      case "route":
        String? route = call.arguments["route"];
        _onRoute(route);
        return true;
      default:
        return false;
    }
  }
}

class CuidaiPlugin {
  static const MethodChannel _channelToNative = MethodChannel('cuidai_plugin');
  static const MethodChannel _channelFromNative =
      MethodChannel('cuidai_plugin_native');
  static String schema = "vitat://";

  static void initSDK({required String appName}) async {
    try {
      schema = "${appName.toLowerCase()}://";
      await _channelToNative.invokeMethod('initSDK', {'appName': appName});
    } on PlatformException catch (e) {
      print(e.message);
    }
  }

  static void logout() async {
    try {
      await _channelToNative.invokeMethod('logout');
    } on PlatformException catch (e) {
      print(e.message);
    }
  }

  static void startCommunication(CuidaiCommunication? communication) async {
    if (communication != null) {
      _channelFromNative
          .setMethodCallHandler(communication.nativeMethodCallHandler);
    }
  }

  static void stopCommunication() {
    _channelFromNative.setMethodCallHandler(null);
  }

  static void setUserProperty(String key, {String? value}) async {
    try {
      await _channelToNative
          .invokeMethod('setUserProperty', {"key": key, "value": value});
    } on PlatformException catch (e) {
      print(e.message);
    }
  }

  static void setUserProperties(Map<String, String> properties) async {
    try {
      await _channelToNative.invokeMethod('setUserProperties', {
        "properties": properties,
      });
    } on PlatformException catch (e) {
      print(e.message);
    }
  }

  static void cleanUserProperties() async {
    try {
      await _channelToNative.invokeMethod('cleanUserProperties');
    } on PlatformException catch (e) {
      print(e.message);
    }
  }

  static void route(
      {required String deepLink,
      Map<String, String>? params,
      BuildContext? context}) async {
    try {
      var route = deepLink.startsWith(schema) ? deepLink : "$schema$deepLink";

      if (route.startsWith("${schema}menu")) {
        if (context != null) {
          Map<String, String> headers = Map<String, String>();

          var result = await _channelToNative.invokeMethod('getHeaders')
              as Map<dynamic, dynamic>;

          headers.addAll(Map<String, String>.from(result));

          Navigator.of(context, rootNavigator: true).push(MaterialPageRoute(
              builder: (context) => FoodSubstitutionApp(
                  headers: headers,
                  params: Uri.parse(route).queryParameters,
                  routeCallback: (route) async => {
                        await _channelToNative.invokeMethod('route', {
                          "route": route.startsWith(schema) ? route : "$schema$route",
                          "params": params != null ? params : {}
                        })
                      })));
        }
      } else {
        await _channelToNative.invokeMethod(
            'route', {"route": route, "params": params != null ? params : {}});
      }
    } on PlatformException catch (e) {
      print(e.message);
    }
  }
}
